package dev.shadowsoffire.apotheosis.adventure.commands;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.mojang.brigadier.arguments.ArgumentType;
import com.mojang.brigadier.builder.ArgumentBuilder;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.builder.RequiredArgumentBuilder;
import com.mojang.brigadier.context.CommandContext;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import com.mojang.brigadier.suggestion.SuggestionProvider;
import com.mojang.brigadier.suggestion.SuggestionsBuilder;
import dev.shadowsoffire.apotheosis.adventure.boss.ApothBoss;
import dev.shadowsoffire.apotheosis.adventure.boss.BossRegistry;
import dev.shadowsoffire.apotheosis.adventure.commands.LootifyCommand;
import dev.shadowsoffire.apotheosis.adventure.compat.GameStagesCompat;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;
import java.util.concurrent.CompletableFuture;
import java.util.function.Predicate;
import javax.annotation.Nullable;
import net.minecraft.commands.CommandSourceStack;
import net.minecraft.commands.Commands;
import net.minecraft.commands.SharedSuggestionProvider;
import net.minecraft.commands.arguments.EntityArgument;
import net.minecraft.commands.arguments.ResourceLocationArgument;
import net.minecraft.commands.arguments.coordinates.Vec3Argument;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Position;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.ServerLevelAccessor;
import net.minecraft.world.phys.Vec3;

public class BossCommand {
  public static final SuggestionProvider<CommandSourceStack> SUGGEST_BOSS;
  
  static {
    SUGGEST_BOSS = ((ctx, builder) -> SharedSuggestionProvider.m_82981_(BossRegistry.INSTANCE.getKeys().stream().map(ResourceLocation::toString), builder));
  }
  
  public static void register(LiteralArgumentBuilder<CommandSourceStack> root) {
    LiteralArgumentBuilder<CommandSourceStack> builder = (LiteralArgumentBuilder<CommandSourceStack>)Commands.m_82127_("spawn_boss").requires(c -> c.m_6761_(2));
    builder.then((
        (RequiredArgumentBuilder)Commands.m_82129_("pos", (ArgumentType)Vec3Argument.m_120841_())
        .then(((RequiredArgumentBuilder)Commands.m_82129_("boss", (ArgumentType)ResourceLocationArgument.m_106984_()).suggests(SUGGEST_BOSS)
          .then(Commands.m_82129_("rarity", (ArgumentType)ResourceLocationArgument.m_106984_()).suggests(LootifyCommand.SUGGEST_RARITY)
            .executes(c -> spawnBoss(c, Vec3Argument.m_120844_(c, "pos"), ResourceLocationArgument.m_107011_(c, "boss"), ResourceLocationArgument.m_107011_(c, "rarity")))))
          .executes(c -> spawnBoss(c, Vec3Argument.m_120844_(c, "pos"), ResourceLocationArgument.m_107011_(c, "boss"), null))))
        .executes(c -> spawnBoss(c, Vec3Argument.m_120844_(c, "pos"), null, null)));
    builder.then((
        (RequiredArgumentBuilder)Commands.m_82129_("entity", (ArgumentType)EntityArgument.m_91449_())
        .then(((RequiredArgumentBuilder)Commands.m_82129_("boss", (ArgumentType)ResourceLocationArgument.m_106984_()).suggests(SUGGEST_BOSS)
          .then(Commands.m_82129_("rarity", (ArgumentType)ResourceLocationArgument.m_106984_()).suggests(LootifyCommand.SUGGEST_RARITY)
            .executes(c -> spawnBoss(c, EntityArgument.m_91452_(c, "entity").m_20182_(), ResourceLocationArgument.m_107011_(c, "boss"), ResourceLocationArgument.m_107011_(c, "rarity")))))
          .executes(c -> spawnBoss(c, EntityArgument.m_91452_(c, "entity").m_20182_(), ResourceLocationArgument.m_107011_(c, "boss"), null))))
        .executes(c -> spawnBoss(c, EntityArgument.m_91452_(c, "entity").m_20182_(), null, null)));
    root.then((ArgumentBuilder)builder);
  }
  
  public static int spawnBoss(CommandContext<CommandSourceStack> c, Vec3 pos, @Nullable ResourceLocation bossId, @Nullable ResourceLocation rarityId) {
    Mob bossEntity;
    Entity nullableSummoner = ((CommandSourceStack)c.getSource()).m_81373_();
    Player summoner = (nullableSummoner instanceof Player) ? (Player)nullableSummoner : ((CommandSourceStack)c.getSource()).m_81372_().m_45924_(pos.m_7096_(), pos.m_7098_(), pos.m_7094_(), 64.0D, false);
    if (summoner == null) {
      ((CommandSourceStack)c.getSource()).m_81352_((Component)Component.m_237113_("No available player context!"));
      return -1;
    } 
    ApothBoss boss = (bossId == null) ? (ApothBoss)BossRegistry.INSTANCE.getRandomItem(summoner.f_19796_, summoner.m_36336_(), new Predicate[] { WeightedDynamicRegistry.IDimensional.matches(summoner.m_9236_()), GameStagesCompat.IStaged.matches(summoner) }) : (ApothBoss)BossRegistry.INSTANCE.getValue(bossId);
    if (boss == null) {
      if (bossId == null) {
        ((CommandSourceStack)c.getSource()).m_81352_((Component)Component.m_237113_("Unknown boss: " + bossId));
      } else {
        ((CommandSourceStack)c.getSource()).m_81352_((Component)Component.m_237113_("No bosses available for the current context!"));
      } 
      return -2;
    } 
    if (rarityId != null) {
      DynamicHolder<LootRarity> rarity = RarityRegistry.INSTANCE.holder(rarityId);
      if (!rarity.isBound()) {
        ((CommandSourceStack)c.getSource()).m_81352_((Component)Component.m_237113_("Unknown rarity: " + rarityId));
        return -3;
      } 
      bossEntity = boss.createBoss((ServerLevelAccessor)summoner.m_9236_(), BlockPos.m_274446_((Position)pos), summoner.f_19796_, summoner.m_36336_(), (LootRarity)rarity.get());
    } else {
      bossEntity = boss.createBoss((ServerLevelAccessor)summoner.m_9236_(), BlockPos.m_274446_((Position)pos), summoner.f_19796_, summoner.m_36336_());
    } 
    ((CommandSourceStack)c.getSource()).m_81372_().m_47205_((Entity)bossEntity);
    return 0;
  }
}
